@@ -10,7 +10,10 @@ import re
 import sys
 
 import requests
+from urllib3 import disable_warnings
+from urllib3.exceptions import InsecureRequestWarning
 
+disable_warnings(InsecureRequestWarning)
 
 # define regex to filter invalid XML codes
 # cf https://stackoverflow.com/questions/1707890/fast-way-to-filter-illegal-xml-unicode-chars-in-python
@@ -37,6 +40,7 @@ _illegal_xml_chars_RE = re.compile(u'[%s]' % u''.join(_illegal_ranges))
 DEFAULT_HOST = 'routerlogin.net'
 DEFAULT_USER = 'admin'
 DEFAULT_PORT = 5000
+ALL_PORTS = [(5555, True), (443, True), (5000, False), (80, False)]
 _LOGGER = logging.getLogger(__name__)
 
 
@@ -72,40 +76,68 @@ class Netgear(object):
         ssl=False,
         url=None,
         force_login_v1=False,
-        # not used and only kept for backward compat, v2 being the default now
         force_login_v2=False,
     ):
         """Initialize a Netgear session."""
         if not url and not host and not port:
             url = autodetect_url()
 
-        if url:
-            self.soap_url = url + "/soap/server_sa/"
-        else:
-            if not host:
-                host = DEFAULT_HOST
-            if not port:
-                port = DEFAULT_PORT
-            scheme = "https" if ssl else "http"
-            self.soap_url = "{}://{}:{}/soap/server_sa/".format(
-                scheme,
-                host,
-                port
-            )
-
+        if not host:
+            host = DEFAULT_HOST
+        if not port:
+            port = DEFAULT_PORT
         if not user:
             user = DEFAULT_USER
 
         self.username = user
         self.password = password
+        self.url = url
+        self.host = host
         self.port = port
+        self.ssl = ssl
         self.force_login_v1 = force_login_v1
+        self.force_login_v2 = force_login_v2
         self.cookie = None
         self.config_started = False
         self._logging_in = False
+        self._login_version = 2
 
         self._info = None
 
+    @property
+    def soap_url(self):
+        """SOAP url to connect to the router."""
+        if self.url:
+            return self.url + "/soap/server_sa/"
+
+        scheme = "https" if self.ssl else "http"
+        return "{}://{}:{}/soap/server_sa/".format(
+            scheme, self.host, self.port)
+
+    def login_try_port(self):
+        # first try the currently configured port-ssl combination
+        current_port = (self.port, self.ssl)
+        if self.login():
+            return True
+
+        ports = ALL_PORTS.copy()
+        if current_port in ports:
+            ports.remove(current_port)
+
+        for port in ports:
+            self.port = port[0]
+            self.ssl = port[1]
+            if self.login():
+                _LOGGER.info("Login succeeded using non default port "
+                             "'%i' and ssl '%r'.", self.port, self.ssl)
+                return True
+
+        # reset original port-ssl
+        self.port = current_port[0]
+        self.ssl = current_port[1]
+        _LOGGER.error("login using all known port-ssl combinations failed.")
+        return False
+
     def login(self):
         """
         Login to the router.
@@ -121,18 +153,29 @@ class Netgear(object):
         # one login attempt has been made for v1
         self.cookie = None
 
-        if not self.force_login_v1:
-            v2_result = self.login_v2()
-            if v2_result:
-                self._logging_in = False
-                return v2_result
+        # if a force option is given always start with that method
+        if self.force_login_v1:
+            self._login_version = 1
+        if self.force_login_v2:
+            self._login_version = 2
 
-        v1_result = self.login_v1()
+        login_methods = [self.login_v1, self.login_v2]
+        for idx in range(0, len(login_methods)):
+            login_version = (idx + self._login_version) % len(login_methods)
+            login_method = login_methods[login_version-1]
+            if login_method():
+                # login succeeded, next time start with this login method
+                self._logging_in = False
+                self._login_version = login_version
+                return True
+
+        # login failed, next time start trying with the other login method
         self._logging_in = False
-        return v1_result
+        self._login_version = self._login_version + 1
+        return False
 
     def login_v2(self):
-        _LOGGER.debug("Login v2")
+        _LOGGER.debug("Login v2, port '%i', ssl, '%r'", self.port, self.ssl)
 
         success, response = self._make_request(
             SERVICE_DEVICE_CONFIG,
@@ -158,7 +201,7 @@ class Netgear(object):
         return True
 
     def login_v1(self):
-        _LOGGER.debug("Login v1")
+        _LOGGER.debug("Login v1, port '%i', ssl, '%r'", self.port, self.ssl)
 
         body = LOGIN_V1_BODY.format(username=self.username,
                                     password=self.password)
@@ -177,7 +220,7 @@ class Netgear(object):
         if self.get_info(use_cache=False) is None:
             return False
 
-        return success
+        return True
 
     def get_info(self, use_cache=True):
         """
@@ -538,18 +581,27 @@ class Netgear(object):
                 # or the IP-bound (?) session expired (v1)
                 self.cookie = None
 
-                _LOGGER.warning("Unauthorized response, let's login and retry...")
-                if self.login():
-                    # reset headers with new cookie first
-                    headers = self._get_headers(service, method, need_auth)
-                    response = requests.post(self.soap_url, headers=headers,
-                                             data=message, timeout=30, verify=False)
+                _LOGGER.debug("Unauthorized response, "
+                              "let's login and retry...")
+                if not self.login():
+                    _LOGGER.error("Unauthorized response, re-login failed")
+                    return False, response
+
+                # reset headers with new cookie first and re-try
+                headers = self._get_headers(service, method, need_auth)
+                response = requests.post(self.soap_url, headers=headers,
+                                         data=message, timeout=30,
+                                         verify=False)
 
             success = _is_valid_response(response)
-
-            if not success:
-                _LOGGER.error("Invalid response")
-                _LOGGER.debug("%s\n%s\n%s", response.status_code, str(response.headers), response.text)
+            if not success and not self._logging_in:
+                if _is_unauthorized_response(response):
+                    _LOGGER.error("Unauthorized response, "
+                                  "after seemingly successful re-login")
+                else:
+                    _LOGGER.error("Invalid response")
+                    _LOGGER.debug("%s\n%s\n%s", response.status_code,
+                                  str(response.headers), response.text)
 
             return success, response
 
